﻿Imports System.Net, System.Net.Sockets, System.IO, System.Threading
Imports System.Text.Encoding

'Imports PokemonDSLib, PokemonDSLib.PokemonLib

Module Module1

    Dim serv As Socket
    Dim bin, pkm As Byte()
    Dim pathtopoke As String = ""

    Dim alreadysentDic As Dictionary(Of String, Boolean) = New Dictionary(Of String, Boolean)

    Structure ReqStruct
        Dim sock As Socket
        Dim req As Request
    End Structure

    Sub Main(ByVal args As String())

        If args.Length() > 0 Then
            If args.Length() > 1 Then
                Console.WriteLine("Too many parameters ! Using the first one as the Pokémon to send.")
                Console.WriteLine()
            End If

            pathtopoke = args(0)
        End If

        Do Until File.Exists(pathtopoke)
            If pathtopoke <> "" Then
                Console.WriteLine("The file doesn't exist !")
            End If

            Console.WriteLine("Please enter the path of the file to send :")
            pathtopoke = Console.ReadLine()
        Loop

        Console.Write("Using file ")
        Console.ForegroundColor = ConsoleColor.White
        Console.WriteLine(pathtopoke)
        Console.ResetColor()
        Console.WriteLine()

        Select Case Path.GetExtension(pathtopoke).ToLower()
            Case ".pkm"
                pkm = File.ReadAllBytes(pathtopoke)
                Console.WriteLine("Encoding .pkm...")
                bin = Pokemon.EncryptPokemon(pkm.Clone())

            Case ".bin"
                bin = File.ReadAllBytes(pathtopoke)
                Console.WriteLine("Decoding .bin...")
                pkm = Pokemon.DecryptPokemon(bin.Clone())

            Case Else
                Console.ForegroundColor = ConsoleColor.Red
                Console.WriteLine("Error : please use either a .bin or a .pkm file with this sender.")
                Console.ResetColor()
                Exit Sub
        End Select

        If pkm.Length < 236 Then
            ReDim Preserve pkm(235)
        ElseIf pkm.Length > 236 Then
            Console.ForegroundColor = ConsoleColor.Red
            Console.WriteLine("Error : incorrect file size !")
            Console.ResetColor()
            Exit Sub
        End If

        Dim binEnd(55) As Byte
        Array.Copy(pkm, 8, binEnd, 0, 2) 'ID
        binEnd(2) = If((pkm(64) And 4) > 0, 3, (pkm(64) And 2) + 1) 'Gender
        binEnd(3) = pkm(140) 'Level
        Array.Copy(New Byte() {&H1, &H0, &H3, &H0, &H0, &H0, &H0, &H0}, 0, binEnd, 4, 8) 'Requesting Bulba, either, any
        Array.Copy(pkm, &H68, binEnd, 32, 16) 'OT name
        Array.Copy(pkm, &HC, binEnd, 48, 2) 'OT ID
        ReDim Preserve bin(291)
        Array.Copy(binEnd, 0, bin, 236, 56)

        serv = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
        serv.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, 1)

        serv.Bind(New IPEndPoint(IPAddress.Any, 80))
        serv.Listen(5)

        Dim t As Thread

        Console.WriteLine()
        Console.WriteLine("Ready !")
        Console.WriteLine("Waiting for connexion...")
        Console.WriteLine()

        While True
            t = New Thread(AddressOf asyncReq)
            t.SetApartmentState(ApartmentState.STA)
            t.Start(serv.Accept())
        End While

        Console.Write("Exiting...")
        serv.Shutdown(SocketShutdown.Both)
    End Sub

    Private Sub asyncReq(ByVal sk As Socket)
        Dim ansStr As String
        Dim ans As Byte() = {}
        Dim alreadysent As Boolean
        Dim reqst As ReqStruct

        Try
            reqst = GetReq(sk)
        Catch ex As Exception
            Console.ForegroundColor = ConsoleColor.Red
            Console.WriteLine("Got wrong request, probably not from a DS. Error was : " & ex.Message())
            Console.ResetColor()
            Console.WriteLine()
            Exit Sub
        End Try

        Dim sock As Socket = reqst.sock
        Dim req As Request = reqst.req
        Dim pid As String = req.getvars("pid")

        If alreadysentDic.Keys.Contains(pid) Then
            alreadysent = alreadysentDic(pid)
        Else
            alreadysentDic.Add(pid, False)
            alreadysent = False
        End If

        Console.Write("Client ")
        Console.ForegroundColor = ConsoleColor.DarkCyan
        Console.WriteLine(sock.RemoteEndPoint.ToString())
        Console.ResetColor()
        Console.WriteLine("Requesting : " & req.RequestedPage())
        Console.WriteLine()

        If req.getvars.Count = 1 Then
            ansStr = RandStr(32)
        Else
            Select Case req.action
                Case "info"
                    ans = New Byte() {1, 0}

                Case "setProfile"
                    ans = New Byte() {0, 0, 0, 0, 0, 0, 0, 0}

                Case "result"
                    If alreadysent Then
                        ans = New Byte() {4, 0}
                        alreadysentDic.Remove(pid)
                    Else
                        ans = bin
                    End If

                Case "get"
                    ans = bin

                Case "return", "delete"
                    ans = New Byte() {1, 0}
                    Console.Write("Saying 'Bye' to ")
                    Console.ForegroundColor = ConsoleColor.DarkRed
                    Console.Write(Path.GetFileNameWithoutExtension(pathtopoke))
                    Console.ResetColor()
                    Console.WriteLine(" !")
                    Console.WriteLine()
                    alreadysentDic(pid) = True
            End Select

            ansStr = GetEncoding("iso-8859-1").GetString(ans)
        End If

        SendResp(sock, ansStr)
    End Sub

    Private Function GetReq(ByRef sock As Socket) As ReqStruct
        'Dim sock As Socket = serv.Accept()
        Dim a(sock.ReceiveBufferSize - 1) As Byte

        Dim cnt As Integer = sock.Receive(a, sock.ReceiveBufferSize, SocketFlags.None)
        ReDim Preserve a(cnt - 1)

        Dim data As String = GetEncoding("iso-8859-1").GetString(a)

        Dim ret As ReqStruct = New ReqStruct
        ret.req = New Request(data)
        ret.sock = sock

        Return ret
    End Function

    Private Function SendResp(ByRef sock As Socket, ByVal data As Object) As Integer
        Dim resp As Response = If(TypeOf data Is Response, data, New Response(data))

        Return sock.Send(GetEncoding("iso-8859-1").GetBytes(resp.ToString))
    End Function

    Private Function RespFromServ(ByVal req As Request) As Response
        Dim s As Socket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
        s.Connect("207.38.11.146", 80)
        s.Send(GetEncoding("iso-8859-1").GetBytes(req.ToString))

        Dim a(8191) As Byte

        Dim cnt As Integer = s.Receive(a, 8192, SocketFlags.None)
        ReDim Preserve a(cnt - 1)

        Dim data As String = GetEncoding("iso-8859-1").GetString(a)

        Return New Response(data)
    End Function

    Private Function RandStr(ByVal len As Integer) As String
        Dim pop As String = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
        RandStr = ""
        Dim rnd As Random = New Random()

        For i = 1 To len
            RandStr &= pop(rnd.Next(0, pop.Length))
        Next
    End Function

    'Private Sub serverResp()
    'Dim req As ReqStruct = GetReq()
    'Dim resp As Response = RespFromServ(req.req)
    'SendResp(req.sock, resp)
    'End Sub

End Module

Class Request
    Public action, page As String
    Public getvars As Dictionary(Of String, String)

    Public Sub New()
        action = Nothing
        page = Nothing
        getvars = New Dictionary(Of String, String)
    End Sub

    Public Sub New(ByVal h As String)
        If Not h.StartsWith("GET") Then
            Throw New Exception("Not a DS header !")
        End If

        Dim request As String = ""

            request = h.Substring(h.IndexOf("/pokemondpds/") + 13, h.IndexOf("HTTP/1.1") - h.IndexOf("/pokemondpds/") - 14)

            page = Left(request, request.IndexOf("?"))
            action = request.Substring(request.IndexOf("/") + 1, request.IndexOf(".asp?") - request.IndexOf("/") - 1)

            Dim vars As New Dictionary(Of String, String)

            For Each i As String In request.Substring(request.IndexOf("?") + 1).Split("&")
                vars.Add(Left(i, i.IndexOf("=")), i.Substring(i.IndexOf("=") + 1))
            Next i

            getvars = vars
    End Sub

    Public Overrides Function ToString() As String
        Dim request As String = page & "?"

        For Each i As KeyValuePair(Of String, String) In getvars
            request &= i.Key & "=" & i.Value & "&"
        Next

        request = Left(request, request.Length - 1)

        Return "GET /pokemondpds/" & request & " HTTP/1.1\r\n" & _
               "Host: gamestats2.gs.nintendowifi.net\r\nUser-Agent: GameSpyHTTP/1.0\r\n" & _
               "Connection: close\r\n\r\n"
    End Function

    Public Function RequestedPage() As String
        RequestedPage = page & "?"

        For Each i As KeyValuePair(Of String, String) In getvars
            RequestedPage &= i.Key & "=" & i.Value & "&"
        Next

        RequestedPage = Left(RequestedPage, RequestedPage.Length - 1)
    End Function
End Class

Class Response
    Dim data As String
    Dim headers As List(Of String)

    Dim p3p, server, sname, cookie As String
    Dim len As Integer

    Public Sub New(ByVal h As String)
        If Not h.StartsWith("HTTP/1.1") Then
            data = h
            Return
        End If

        headers = h.Split(vbCrLf).ToList()

        Dim line As String

        Do
            line = headers(0)
            headers.RemoveAt(0)

            If line.StartsWith("P3P") Then p3p = line.Substring(line.IndexOf(": ") + 2)
            If line.StartsWith("cluster-server") Then server = line.Substring(line.IndexOf(": ") + 2)
            If line.StartsWith("X-Server-") Then sname = line.Substring(line.IndexOf(": ") + 2)
            If line.StartsWith("Content-Length") Then len = Int(line.Substring(line.IndexOf(": ") + 2))
            If line.StartsWith("Set-Cookie") Then cookie = line.Substring(line.IndexOf(": ") + 2)
        Loop Until line = ""

        data = String.Join(vbCrLf, headers.ToArray)
    End Sub

    Public Overrides Function ToString() As String
        Dim months As String() = {"???", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"}
        Dim days As String() = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"}

        Dim t As Date = Now()

        Return "HTTP/1.1 200 OK" & vbCrLf & _
               "Date: " & days(t.DayOfWeek()) & ", " & t.Day() & " " & t.Month() & " " & t.Year() & _
                    " " & t.Hour() & ":" & t.Minute() & ":" & t.Second() & " GMT" & vbCrLf & _
               "Server: Microsoft-IIS/6.0" & vbCrLf & _
               "P3P: CP='NOI ADMa OUR STP'" & vbCrLf & _
               "cluster-server: aphexweb3" & vbCrLf & _
               "X-Server-Name: AW4" & vbCrLf & _
               "X-Powered-By: ASP.NET" & vbCrLf & _
               "Content-Length: " & data.Length & vbCrLf & _
               "Content-Type: text/html" & vbCrLf & _
               "Set-Cookie: ASPSESSIONIDQCDBDDQS=JFDOAMPAGACBDMLNLFBCCNCI; path=/" & vbCrLf & _
               "Cache-control: private" & vbCrLf & vbCrLf & data
        '"Content-type: application/octet-stream" & vbCrLf & _
    End Function

    Public Function getpkm() As String()
        Dim all As List(Of String) = New List(Of String)
        Dim data_tmp As String = data
        Dim result As String = ""

        While data_tmp.Length > 0
            result = Left(data, 292)
            data = data.Substring(292)
            all.Add(Left(result, 136))
        End While

        Return all.ToArray()
    End Function
End Class